#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  5 15:06:20 2021

@author: tucy
"""

import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--analyzed_property', type=str, default="tadf")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")

args = parser.parse_args()

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX


print("analyzed_property: ", args.analyzed_property, type(args.analyzed_property) )
analyzed_property = args.analyzed_property


import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
#import tensorflow as tf

import deepchem as dc
import numpy as np
import pandas as pd
#import tempfile
from sklearn.ensemble import RandomForestRegressor

from rdkit import Chem
#from rdkit.Chem import Draw, AllChem
#from rdkit.Chem import rdDepictor

from sklearn.model_selection import GridSearchCV
#from sklearn.metrics import mean_squared_error
from sklearn.model_selection import cross_val_score
import joblib
from sklearn.decomposition import PCA
from sklearn.preprocessing import MinMaxScaler
import matplotlib.pyplot as plt

#pd.set_option('display.max_columns', 100)
#pd.set_option('max_colwidth', 400)

from sklearn.impute import SimpleImputer

import subprocess


#------------------------------------------------------------------------------
def display_scores(scores):
    print("Scores:", scores)
    print("Mean:", scores.mean())
    print("Standard deviation:", scores.std())


def load_mols_in_sdf(sdf_path='./data/Pick_mols.sdf'):
    #-------- loading  sdf file ---------------------------------------------------
    sdsuppl = Chem.SDMolSupplier(sdf_path)
    mols = [x for x in sdsuppl if x is not None]
    smiles = [Chem.MolToSmiles(mol) for mol in mols]
    print("-"*60)  
    print( "number of molecules is ", len(mols) )
    
    return mols, smiles


def impute_dataset(X, strategy="median"):
    imputer = SimpleImputer(strategy=strategy)
    imputer.fit(X)
    X_imputed = imputer.transform(X)
    print("type(X_imputed) is ", type(X_imputed))
    
    return X_imputed 


def load_nlo_csv_creat_properties_list(filename='./data/nlo-summary.csv'):
    #---load properties summary file-----------------------------------------------
    df_props = pd.read_csv(filename)
    
    df_props["beta_x"]  = (df_props["bxxx"]  +  df_props["bxyy"] + df_props["bxzz"] )
    df_props["beta_y"]  = (df_props["bxxy"]  +  df_props["byyy"] + df_props["byzz"] )
    df_props["beta_z"]  = (df_props["bxxz"]  +  df_props["byyz"] + df_props["bzzz"] )
    df_props["beta_total"]  = np.sqrt(df_props["beta_x"]**2 + df_props["beta_y"]**2 + df_props["beta_z"]**2)
    
    properties = df_props["beta_total"].tolist()

    print("-"*60) 
    print("properties set is")
    print(properties)
    
    print("-"*60) 
    print( "df_props.beta_total.describe()" )
    print( df_props["beta_total"].describe() )
    
    return properties


def load_tadf_csv_creat_properties_list(filename='./data/tadf-summary.csv'):
    #---load properties summary file-----------------------------------------------
    df_props = pd.read_csv(filename)
    print(df_props.head())
    df_props_num = df_props.drop("compound", axis=1)
    print(df_props_num.head())
    imputer = SimpleImputer(strategy="median")
    imputer.fit(df_props_num)
    props_imputed = imputer.transform(df_props_num)
    print("type(props_imputed) is ", type(props_imputed))
    
    column_names = ["T1_en", "S1_en", "S1_freq", "T2_en", "S2_en", "S2_freq",\
                    "T3_en","S3_en","S3_freq",\
                        # "T4_en","S4_en","S4_freq",\
                        # "T5_en","S5_en","S5_freq","T6_en","S6_en","S6_freq",\
                        #     "T7_en","S7_en","S7_freq","T8_en","S8_en","S8_freq",\
                        #     "T9_en","S9_en","S9_freq","T10_en","S10_en","S10_freq"
                    ]
    
    df_props_imputed = pd.DataFrame(data=props_imputed, columns=column_names)
    
    df_props_imputed["deltaEST"]  = df_props_imputed["S1_en"]  -  df_props_imputed["T1_en"] 
    
    properties = df_props_imputed["deltaEST"].tolist()

    print("-"*60) 
    print("properties set is")
    print(properties)
    
    print("-"*60) 
    print( "df_props_imputed.deltaEST.describe()" )
    print( df_props_imputed["deltaEST"].describe() )
    
    return properties


def load_tpa_csv_creat_properties_list(filename='./data/tpa-summary.csv'):
    #---load properties summary file-----------------------------------------------
    df_props = pd.read_csv(filename)
    print(df_props.head())
    df_props_num = df_props.drop("compound", axis=1)
    print(df_props_num.head())
    imputer = SimpleImputer(strategy="median")
    imputer.fit(df_props_num)
    props_imputed = imputer.transform(df_props_num)
    print("type(props_imputed) is ", type(props_imputed))
    
    column_names = ["1_En", "1_lsigma", "1_csigma", "2_En", "2_lsigma", "2_csigma",\
                    "3_En", "3_lsigma", "3_csigma", "4_En", "4_lsigma", "4_csigma",\
                        "5_En", "5_lsigma", "5_csigma", "6_En", "6_lsigma", "6_csigma"]
    
    df_props_imputed = pd.DataFrame(data=props_imputed, columns=column_names)
    
    properties = df_props_imputed["1_lsigma"].tolist()

    print("-"*60) 
    print("properties set is")
    print(properties)
    
    print("-"*60) 
    print( "df_props_imputed.1_lsigma.describe()" )
    print( df_props_imputed["1_lsigma"].describe() )
    
    return properties


def creat_ECFP_RDKITDES_CM_features_dataset(mols, smiles, properties, is_compute_CM=True):   
    #--------- ECFP ---------------------------------------------------
    featurizer = dc.feat.CircularFingerprint(size=2048)
    ecfp = featurizer.featurize(smiles)
    dataset = dc.data.NumpyDataset(X=ecfp, y=np.array(properties))
    print("ecfp.shape is ", ecfp.shape )
    print("len(dataset) is ", len(dataset) ) 
    
    #-------------------------------------------------------------------
    #------- RDKitDescriptors ------------------------------------------
    rdkit_featurizer = dc.feat.RDKitDescriptors()
    rdkit_des_features = rdkit_featurizer(smiles)
    dataset1 = dc.data.NumpyDataset(X=rdkit_des_features, y=np.array(properties))
    print("len(dataset1) is ", len(dataset1) ) 
    
    print("-"*60) 
    print("RDKit Descriptors and values for a compound.") 
    for feature, descriptor in zip(rdkit_des_features[0][:], rdkit_featurizer.descriptors):
        print(descriptor, feature)
    
    print('The number of sample present is: ', len(rdkit_des_features))
    print('The number of descriptors present is: ', len(rdkit_des_features[0]))
    
    print("rdkit_des_features.shape is ", rdkit_des_features.shape)
    
    #------- WeaveFeaturizer and MolGraphConvFeaturizer
    
    #------------------------------------------------------------------------------
    #------- CoulombMatrix --------------------------------------------------------
    if (is_compute_CM == True):
        coulomb_mat = dc.feat.CoulombMatrix(max_atoms=100)
        coulomb_mat_features = coulomb_mat(mols)
        
        print("type(coulomb_mat_features) ", type(coulomb_mat_features))
        print("coulomb_mat_features.shape is ", coulomb_mat_features.shape)
        
        coulomb_mat_features = coulomb_mat_features.reshape(coulomb_mat_features.shape[0],
                                                            coulomb_mat_features.shape[1]*coulomb_mat_features.shape[2])
        
        print("coulomb_mat_features.shape is ", coulomb_mat_features.shape)
        
        dataset2 = dc.data.NumpyDataset(X=coulomb_mat_features, y=np.array(properties))
        print("len(dataset2) is ", len(dataset2) ) 
        
        return  dataset, dataset1, dataset2
    else:
        return  dataset, dataset1


def creat_smiles_properties_csv_file(smiles, properties, file_path='./data/smiles_properties.csv'):
    #----------- CSV DataLoader ---------------------------------------------------
    # make a dataframe object for creating a CSV file
    df = pd.DataFrame(list(zip(smiles, properties)), columns=["SMILES", "property"])
    print(df)
    tmp_path = file_path
    with open(tmp_path, 'w') as tmp:
        df.to_csv(tmp)



def gridsearchcv_evaluate_model(dataset, model_path=None, verbs=False):
    print("len(dataset) is ", len(dataset) )
    
    X_train, y_train = dataset.X, dataset.y
    
    param_grid = [
        {'n_estimators': [3, 10, 30, 100], 
         'criterion': ["mse", "mae"],
         'max_depth': [2, 5, 10, 50],
         'max_features': ["auto", "sqrt", "log2"]},
        {'bootstrap': [False], 
         'n_estimators': [3, 10, 30, 100], 
         'criterion': ["mse", "mae"],
         'max_depth': [2, 5, 10, 50],
         'max_features': ["auto", "sqrt", "log2"]},
    ]
    
    forest_reg = RandomForestRegressor(random_state=0)
    
    grid_search = GridSearchCV(forest_reg, 
                               param_grid, 
                               cv=5,
                               scoring='neg_mean_squared_error',
                               return_train_score=True,
                               n_jobs=-1,
                               )
    
    grid_search.fit(X_train, y_train)
    
    if verbs == True:
        print('-'*60)
        cvres = grid_search.cv_results_
        for mean_score, params in zip(cvres["mean_test_score"], cvres["params"]):
            print(np.sqrt(-mean_score), params)
    
    print('-'*60)
    print("grid_search.best_params_ is: \n", grid_search.best_params_)
    print("grid_search.best_estimator_ is: \n", grid_search.best_estimator_)
    
    best_rgr = grid_search.best_estimator_
    
    best_rgr.fit(X_train, y_train)
    
    
    scores = cross_val_score(best_rgr, 
                             X_train, 
                             y_train,
                             scoring="neg_mean_squared_error", 
                             cv=5)
    tree_rmse_scores = np.sqrt(-scores)
    
    print('-'*60)
    display_scores(tree_rmse_scores)
    
    if model_path is not None:
        joblib.dump(best_rgr, model_path)
    #joblib.dump(best_rgr, "./models/best_rgr.pkl")
    

def pca_decomposition_visualize(dataset, figname1=None, xlims=None, ylims=None):
    #-------PCA for visualization -------------------------------------------------
    X = dataset.X.copy()
    y = dataset.y.copy()
    pca = PCA(n_components=2, svd_solver='auto')
    X_pca = pca.fit_transform(X)
    
    mm_scaler = MinMaxScaler()
    X_pca_scaled = mm_scaler.fit_transform(X_pca)
    
    print('-'*60)
    print(pca.explained_variance_ratio_)
    print(pca.singular_values_)
    print(X.shape)
    print(X_pca.shape)
    
    
    df_pca = pd.DataFrame(data=X_pca_scaled, columns=["1st-component", "2nd-component"])
    df_pca["property"] = y
    
    ax1 = df_pca.plot(kind="scatter", 
                x="1st-component", 
                y="2nd-component", 
                alpha=0.4,
                #label="prop", 
                figsize=(10,6),
                c="property", 
                cmap=plt.get_cmap("jet"), 
                colorbar=True,
    )
    #plt.legend()
    
    if xlims is not None:
        left, right = xlims
        plt.xlim(left, right)
        ax1.set_xticks([left, right])
    
    if ylims is not None:
        bottom, top = ylims
        plt.ylim(bottom, top)    
        ax1.set_yticks([bottom, top])
    
    plt.xlabel('1st-component')
    plt.ylabel('2nd-component')
    
    fig1 = ax1.get_figure() 
    
    fig1.savefig(figname1)
    
    #plt.show()
    #plt.pause(10)
    plt.close('all')
    



#----MAIN Function begins -----------------------------------------------------
#------------------------------------------------------------------------------

#mols, smiles = load_mols_in_sdf(sdf_path='./data/Pick_mols.sdf')

GXXXX_PXXXXrev_sdf_path = './project/'  +  GXXXX  + '/'    +  PXXXX   +  'rev.sdf'
mols, smiles = load_mols_in_sdf(sdf_path=GXXXX_PXXXXrev_sdf_path) 

#properties = load_nlo_csv_creat_properties_list(filename='./data/nlo-summary.csv')
#properties = load_tadf_csv_creat_properties_list(filename='./data/tadf-summary.csv')

if (analyzed_property == "tadf") or (analyzed_property == "TADF"):

    GXXXX_tadf_summary_PXXXXrev_csv_path = './project/'   +  GXXXX  + '/tadf_summary_'  +  PXXXX  +  '.csv'
    properties = load_tadf_csv_creat_properties_list(filename=GXXXX_tadf_summary_PXXXXrev_csv_path)

elif (analyzed_property == "nlo") or (analyzed_property == "NLO"):
    
    GXXXX_nlo_summary_PXXXXrev_csv_path = './project/'   +  GXXXX  + '/nlo_summary_'  +  PXXXX  +  '.csv'
    properties = load_nlo_csv_creat_properties_list(filename=GXXXX_nlo_summary_PXXXXrev_csv_path)

else:
    print("undefined property.")
    print("analyzed_property: tadf [TADF] or nlo [NLO] ")
    exit(1)



#dataset, dataset1, dataset2 =  creat_ECFP_RDKITDES_CM_features_dataset(mols, smiles, properties)
dataset, dataset1 =  creat_ECFP_RDKITDES_CM_features_dataset(mols, 
                                                             smiles, 
                                                             properties, 
                                                             is_compute_CM=False
                                                             )

X_imputed = impute_dataset(dataset.X, strategy="median")
X1_imputed = impute_dataset(dataset1.X, strategy="median")
# X2_imputed = impute_dataset(dataset2.X, strategy="median")

dataset_imputed = dc.data.NumpyDataset(X=X_imputed, y=np.array(properties)) 
dataset1_imputed = dc.data.NumpyDataset(X=X1_imputed, y=np.array(properties)) 
# dataset2_imputed = dc.data.NumpyDataset(X=X2_imputed, y=np.array(properties)) 

creat_smiles_properties_csv_file(smiles, 
                                 properties,
                                 file_path='./project/%s/smiles_properties_%s.csv'%(GXXXX, PXXXX)
                                 )

call_mkdir_GXXXX_models_dir = 'mkdir  ./project/'  +  GXXXX  +  '/models/  2>/dev/null '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_models_dir )
print("mkdir  ./project/%s/models/.  status, output = "%(GXXXX), status, output)

call_mkdir_GXXXX_models_ECFP_dir = 'mkdir  ./project/'  +  GXXXX  +  '/models/ECFP/  2>/dev/null '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_models_ECFP_dir )
print("mkdir  ./project/%s/models/ECFP/.  status, output = "%(GXXXX), status, output)

call_mkdir_GXXXX_models_RDKIT_DES_dir = 'mkdir  ./project/'  +  GXXXX  +  '/models/RDKIT_DES/  2>/dev/null '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_models_RDKIT_DES_dir )
print("mkdir  ./project/%s/models/RDKIT_DES/.  status, output = "%(GXXXX), status, output)

# status, output = subprocess.getstatusoutput('mkdir  ./project/G0000/models/CM/  2>/dev/null ')
# print("mkdir  ./project/G0000/models/CM/.  status, output = ", status, output)


#------------------------------------------------------------------------------
print("The ML models are stored at: " + "./project/"  +  GXXXX + "/models/" + "\n" )
print("ECFP model", "-"*60) 
rf_reg_ECFP = gridsearchcv_evaluate_model(dataset, 
                                          model_path='./project/%s/models/ECFP/rf_reg_model_ECFP.pkl'%GXXXX
                                          )


print("RDKIT_DES model", "-"*60) 
rf_reg_RDKIT_DES = gridsearchcv_evaluate_model(dataset1_imputed, 
                                                model_path='./project/%s/models/RDKIT_DES/rf_reg_model_RDKIT_DES.pkl'%GXXXX
                                                )


# print("CM model", "-"*60) 
# rf_reg_CM = gridsearchcv_evaluate_model(dataset2, 
#                                         model_path='./project/G0000/models/CM/rf_reg_model_CM.pkl'
#                                         )

#------------------------------------------------------------------------------

call_mkdir_GXXXX_images_dir = 'mkdir  ./project/'  +  GXXXX   + '/images/  2>/dev/null '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_images_dir )
print("mkdir  ./project/%s/images/.  status, output = "%GXXXX, status, output)

print("The PCA decomposition of X vector vs property is stored at: ")
print( "./project/%s/images/"%GXXXX )

fignamea = './project/%s/images/foo-props-pca-ECFP-1.png'%GXXXX
print("save figure\n %s \n" %(fignamea))
pca_decomposition_visualize(dataset, figname1=fignamea)


fignameb = './project/%s/images/foo-props-pca-RDKITDES-1.png'%GXXXX
print("save figure\n %s \n" %(fignameb))
pca_decomposition_visualize(dataset1_imputed, figname1=fignameb)


# fignamec = './project/G0000/images/foo-props-pca-CM-1.png'
# print("save figure\n %s \n" %(fignamec))
# pca_decomposition_visualize(dataset2, 
#                             figname1=fignamec, 
#                             #xlims=(0.0, 0.10), 
#                             #ylims=(0.0, 0.05),
#                             )


print("\n### End of program: PG010!\n")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 25 09:42:45 2021

Murcko Decomposition

@author: tucy
"""


import sys 
import os
#os.chdir('../')
script_wkdir = os.getcwd()
sys.path.append(script_wkdir)


import pandas as pd
# import numpy as np
from rdkit import Chem
# from rdkit.Chem import AllChem
# from rdkit.Chem import Draw
from rdkit.Chem.Scaffolds import MurckoScaffold
from salam.module_MD.D_Pi_A_Enumeration import draw_mols
from salam.module_MD.D_Pi_A_Enumeration import delete_repeated_smiles_of_mols


import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")
parser.add_argument('--num_high_freqs', type=int, default=9)

args = parser.parse_args()

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print("args.num_high_freqs= ", args.num_high_freqs, ", type(args.num_high_freqs)= ", type(args.num_high_freqs))
NUM_HIGH_FREQS = args.num_high_freqs


#---------------------------------------------------------------------
mols_suppl = Chem.SDMolSupplier('./project/%s/%s.sdf'%(GXXXX, GXXXX))

mols_sup = [x for x in mols_suppl if x is not None]
print("len(mols_sup) = ", len(mols_sup))

mols_origin = delete_repeated_smiles_of_mols(mols_sup)
print("After delete_repeated_smiles_of_mols")
print("len(mols_origin) = ", len(mols_origin))

mols = []
cores = []
fws = []


##---- Delete AtomValenceException, if error occurs -----------------
# mols_origin_cp = mols_origin.copy()
# pop_idxs = []
# print("# Delete AtomValenceException, if error occurs.")
# for idx, mol, in zip(range(len(mols_origin_cp)), mols_origin_cp):
#     try:
#         core = MurckoScaffold.GetScaffoldForMol(mol)
#         fw = MurckoScaffold.MakeScaffoldGeneric(core)
#     except Exception as e:
#         # 访问异常的错误编号和详细信息
#         print(e.args)
#         print(str(e))
#         print(repr(e))
#         print("To be poped index is ", idx)
#         pop_idxs.append(idx)

# print("len(pop_idxs) = ", len(pop_idxs))

# if len(pop_idxs) > 0:
#     for idx in sorted(pop_idxs, reverse=True):
#         print("delete molecule: %5d" %idx )
#         mols_origin.pop(idx)
# 
# print("len(mols_origin) = ", len(mols_origin))


for mol in mols_origin:
    core = MurckoScaffold.GetScaffoldForMol(mol)
    fw = MurckoScaffold.MakeScaffoldGeneric(core)
    if Chem.MolToSmiles(core) != "":
        mols.append(mol)
        cores.append(core)
        fws.append(fw)

        
df = pd.DataFrame(data=zip(mols, cores, fws), columns=["mols", "cores", "fws"])
df["smis"] = df["mols"].apply(lambda x: Chem.MolToSmiles(x) )
df["core_smis"] = df["cores"].apply(lambda x: Chem.MolToSmiles(x) )
df["fw_smis"] = df["fws"].apply(lambda x: Chem.MolToSmiles(x) )

print(df.head())
print(df.shape)

print(df.info() )
print(df.describe() )

print("size of smis = ", len(set(list(df["smis"]))))
print("size of core_smis = ", len(set(list(df["core_smis"]))))
print("size of fw_smis = ", len(set(list(df["fw_smis"]))))

df_core_smis = df["core_smis"].value_counts()
print('-'*60)
print("df.core_smis.value_counts():  1-%d high freqs."%NUM_HIGH_FREQS)
print(df_core_smis.head(NUM_HIGH_FREQS))

core_smis_str = str(df_core_smis.head(NUM_HIGH_FREQS)).split("\n")
core_smis_str1 = [ele.strip().split()[0] for ele in core_smis_str]
core_smis_str1.pop(-1)
core_highfreq_mols = [Chem.MolFromSmiles(smi) for smi in core_smis_str1]

core_freqs = [ele.strip().split()[1] for ele in core_smis_str]
core_freqs.pop(-1)

print("core_freqs \n", core_freqs)


df_fw_smis = df["fw_smis"].value_counts()
print('-'*60)
print("df.fw_smis.value_counts():  1-%d high freqs."%NUM_HIGH_FREQS)
print(df_fw_smis.head(NUM_HIGH_FREQS))

fw_smis_str = str(df_fw_smis.head(NUM_HIGH_FREQS)).split("\n")
fw_smis_str1 = [ele.strip().split()[0] for ele in fw_smis_str]
fw_smis_str1.pop(-1)
fw_highfreq_mols = [Chem.MolFromSmiles(smi) for smi in fw_smis_str1]

fw_freqs = [ele.strip().split()[1] for ele in fw_smis_str]
fw_freqs.pop(-1)

print("fw_freqs \n", fw_freqs)


# draw_mols(mols, 
#           9, 
#           './project/G0000/images/foo001-Murcko-decomposition.png', 
#           molsPerRow=3, 
#           size=(800, 800)
#           )


draw_mols(core_highfreq_mols, 
          len(core_highfreq_mols), 
          './project/%s/images/foo-%s-Murcko-decomp-highfreq-core-mols.png'%(GXXXX, GXXXX), 
          molsPerRow=3, 
          size=(800, 800),
          legends=core_freqs,
          )

draw_mols(fw_highfreq_mols, 
          len(fw_highfreq_mols), 
          './project/%s/images/foo-%s-Murcko-decomp-highfreq-fw-mols.png'%(GXXXX, GXXXX), 
          molsPerRow=3, 
          size=(800, 800),
          legends=fw_freqs,
          )


print("\n### End of program: PG013!\n")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar  1 13:53:00 2021

Using scikit-learn with RDKit

@author: tucy
"""

import sys 
import os
#os.chdir('../')
script_wkdir = os.getcwd()
sys.path.append(script_wkdir)

from salam.module_MD.D_Pi_A_Enumeration import draw_mols
from rdkit import Chem
#from rdkit.Chem import AllChem

# from tqdm import tqdm
# from salam.module_MD.D_Pi_A_Enumeration import try_embedmolecule
from salam.module_MD.D_Pi_A_Enumeration import delete_repeated_smiles_of_mols
from salam.module_MD.D_Pi_A_Enumeration import write_mols_paralell

from salam.module_MD.OrganoMetalics_Enumeration import mutations_CH_aromatic
from salam.module_MD.OrganoMetalics_Enumeration import mutations_CH_aromatic_termino_H
import subprocess

import numpy as np
#from scipy.special import comb

import argparse

parser = argparse.ArgumentParser(description="manual to this script:")

parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")
parser.add_argument('--GXXXXPLUSONE', type=str, default="G0001")

parser.add_argument('--carbon_sub', type=str, default='N' )
#parser.add_argument('--termino_hydrogen_subs', type=list, default=[ '*F',  '*C#N',  '*OC',  '*N(C)C' ], action='append')
parser.add_argument('--termino_hydrogen_subs', action='append')

parser.add_argument('--max_subpos', type=int, default=10)
parser.add_argument('--substi_number_carbon', type=int, default=1)
parser.add_argument('--sample_number_carbon', type=int, default=6)
parser.add_argument('--substi_number_hydrogen', type=int, default=1)
parser.add_argument('--sample_number_hydrogen', type=int, default=6)

parser.add_argument('--is_substi_carbon', type=int, default=1 )
parser.add_argument('--is_substi_hydrogen', type=int, default=1 )

parser.add_argument('--parent_optmols_size', type=int, default=-1 )


args = parser.parse_args()


print("args.carbon_sub: ", args.carbon_sub, type(args.carbon_sub))
CARBON_SUB = args.carbon_sub

print("args.termino_hydrogen_subs: ", args.termino_hydrogen_subs, type(args.termino_hydrogen_subs))
TERMINO_HYDROGEN_SUBS = args.termino_hydrogen_subs

#TERMINO_HYDROGEN_SUBS  = [ ele.strip()  for ele in TERMINO_HYDROGEN_SUBS.strip().split(",") ]
#'max_subpos', 'substi_number_carbon', 'sample_number_carbon', 'substi_number_hydrogen', 'sample_number_hydrogen'

print("args.max_subpos: ", args.max_subpos, type(args.max_subpos ))
print("args.substi_number_carbon: ", args.substi_number_carbon, type(args.substi_number_carbon ))
print("args.sample_number_carbon: ", args.sample_number_carbon, type(args.sample_number_carbon ))
print("args.substi_number_hydrogen: ", args.substi_number_hydrogen, type(args.substi_number_hydrogen ))
print("args.sample_number_hydrogen: ", args.sample_number_hydrogen, type(args.sample_number_hydrogen ))

MAX_SUBPOS = args.max_subpos 
SUBSTI_NUMBER_CARBON = args.substi_number_carbon 
SAMPLE_NUMBER_CARBON = args.sample_number_carbon 
SUBSTI_NUMBER_HYDROGEN = args.substi_number_hydrogen
SAMPLE_NUMBER_HYDROGEN = args.sample_number_hydrogen 

print("args.is_substi_carbon: ", args.is_substi_carbon, type(args.is_substi_carbon))
print("args.is_substi_hydrogen: ", args.is_substi_hydrogen, type(args.is_substi_hydrogen))

IS_SUBSTI_CARBON = bool( args.is_substi_carbon )
IS_SUBSTI_HYDROGEN = bool( args.is_substi_hydrogen )


if TERMINO_HYDROGEN_SUBS is None:
    TERMINO_HYDROGEN_SUBS = [ '*F',  '*C#N',  '*OC',  '*N(C)C' ] 
    

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print(args.GXXXXPLUSONE, type(args.GXXXXPLUSONE))
GXXXXPLUSONE = args.GXXXXPLUSONE


print("args.parent_optmols_size: ", args.parent_optmols_size, type(args.parent_optmols_size))
PARENT_OPTMOLS_SIZE = args.parent_optmols_size 


def random_sampling_mols(All_mols, sample_number=1000):
    #------random sampling 1000 mols------------------------------------------
    print("The length of All_mols is : ", len(All_mols))
    if (sample_number >= 1) & ( len(All_mols) >= sample_number):
        sample_indexs = np.random.choice(len(All_mols), sample_number, replace=False)
        All_mols_sample = [All_mols[idx] for idx in sample_indexs]
        All_mols = All_mols_sample 
        print("After sampling, the length of All_mols is : ", len(All_mols))
    else:
        print("The length of All_mols is  smaller than sample_number: ", len(All_mols) , "   <   ",  sample_number )
    
    return All_mols


#------ main function -----------------------------------------------------
#--------------------------------------------------------------------------

#----- reading opt_mols sdf file -----------------
opt_sdf_path = './project/'  +  GXXXX  +   '/'  +  OXXXX   + '.sdf'
sdsuppl = Chem.SDMolSupplier(opt_sdf_path)
opt_mols = [x for x in sdsuppl if x is not None]

if ( PARENT_OPTMOLS_SIZE > 0 ):
    print("PARENT_OPTMOLS_SIZE = ", PARENT_OPTMOLS_SIZE )
    print("Restrict the size of parent opt_mols by this para.")
    opt_mols = opt_mols[:PARENT_OPTMOLS_SIZE]
else:
    print("All mols in opt_mols will be used.")


print("length of opt_mols is : ", len(opt_mols))


#-----------------------------------------------------------------------------
#---- create combinational compound library --------------------------
#---- via calling mutations_CH_aromatic() or mutations_CH_aromatic_termino_H()

print("\n", '*'*80)

All_mols = []
for opt_mol  in opt_mols:
    
    All_mols +=  [ opt_mol ]
    
    if (IS_SUBSTI_CARBON == True):
        mutation_mols = mutations_CH_aromatic(opt_mol, 
                                              patt=Chem.MolFromSmarts('c[H]'), 
                                              substitute_atom=CARBON_SUB,
                                              substitute_atomnum=7, 
                                              substi_number=SUBSTI_NUMBER_CARBON, 
                                              sample_number=SAMPLE_NUMBER_CARBON, 
                                              #is_showmol=True, 
                                              max_subpos=MAX_SUBPOS
                                              )
        All_mols +=  mutation_mols     
    else:
        print("!!! IS_SUBSTI_CARBON is False. No substitutions occur at site of carbons.")
    

    if (IS_SUBSTI_HYDROGEN == True):     
        for termino_hydrogen_sub in TERMINO_HYDROGEN_SUBS:
            mutation_mols1 = mutations_CH_aromatic_termino_H(opt_mol, 
                                                            patt=Chem.MolFromSmarts('c[H]'), 
                                                            substitute_group=Chem.MolFromSmiles( termino_hydrogen_sub ), 
                                                            substi_number=SUBSTI_NUMBER_HYDROGEN, 
                                                            sample_number=SAMPLE_NUMBER_HYDROGEN, 
                                                            #is_showmol=True,
                                                            max_subpos=MAX_SUBPOS
                                                            )
            All_mols +=  mutation_mols1
    else:
        print("!!! IS_SUBSTI_HYDROGEN is False. No substitutions occur at site of carbons.")
    

print("The length of All_mols is : ", len(All_mols))


draw_mols(All_mols, 
          16, 
          './project/%s/images/foo011--mutation-mols-demo.png'%GXXXX, 
          molsPerRow=4, 
          size=(800, 800)
          )


All_mols  =  delete_repeated_smiles_of_mols(All_mols)
print("After delete_repeated_smiles_of_mols, the length of All_mols is : ", len(All_mols))

#------random sampling 1000 mols-----------------------------------------------------
All_mols = random_sampling_mols(All_mols, sample_number=1000)

cpd_names = [ "cpd-%05d" % (i+1) for i in range(len(All_mols)) ]
for mol, cpd_name in zip(All_mols, cpd_names):
    mol.SetProp("_Name", cpd_name )


#---- write molecules to SD File ------------------------------- 
print("\nWrite to sdf file.\n")

call_mkdir_GXXXXplus1 = 'mkdir  ./project/' +  GXXXXPLUSONE  +  '/  2>/dev/null  '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXXplus1 )
print("mkdir  ./project/GXXXXPLUSONE/;    status, output = ", status, output)

outdir_path = './project/'   +   GXXXXPLUSONE   +  '/separate_mols/'

call_mkdir_GXXXXplus1_separate_mols = 'mkdir   ' +  outdir_path  +  '    2>/dev/null  '
status, output = subprocess.getstatusoutput( call_mkdir_GXXXXplus1_separate_mols )
print("mkdir  ./project/GXXXXPLUSONE/separate_mols/;    status, output = ", status, output)        

status, output = subprocess.getstatusoutput( 'rm   -f    ' + outdir_path + 'cpd-*.sdf   2>/dev/null ' )
print("remove ./project/GXXXXPLUSONE/separate_mols/cpd-*.sdf.  status, output = ", status, output)

print("call write_mols_paralell(), the separate mols are written to dir:  %s"%outdir_path )
write_mols_paralell(All_mols, cpd_names, outfile_path=outdir_path)

print("remove  ./project/GXXXXPLUSONE/GXXXXPLUSONE.sdf")
status, output = subprocess.getstatusoutput( 'rm  -f   ./project/%s'%GXXXXPLUSONE  +  '/%s.sdf'%GXXXXPLUSONE  +  '   2>/dev/null ' )
print("status, output = ", status, output)

cat_sdfs_to_onefile = 'cat   '  +  outdir_path  + 'cpd*.sdf  >>  '  +  './project/%s'%GXXXXPLUSONE  + '/%s.sdf  '%GXXXXPLUSONE
status, output = subprocess.getstatusoutput( cat_sdfs_to_onefile )
print("cat cpd*.sdf to GXXXXPLUSONE.sdf.   status, output = ", status, output)


print("\n### End of program: PG012!\n")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 18 08:45:09 2022

Chem.RDKFingerprint
DataStructs.FingerprintSimilarity

The default set of parameters used by the fingerprinter is:
    - minimum path size: 1 bond - maximum path size: 7 bonds 
    - fingerprint size: 2048 bits 
    - number of bits set per hash: 2 
    - minimum fingerprint size: 64 bits 
    - target on-bit density 0.0

Available similarity metrics include 
Tanimoto, Dice, Cosine, Sokal, Russel, Kulczynski, McConnaughey, and Tversky.

@author: tucy
"""


import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--GXXXX', type=str, default="G0000")

args = parser.parse_args()

print("args.GXXXX = ", args.GXXXX, " , type(args.GXXXX) = ",  type(args.GXXXX))
GXXXX = args.GXXXX


import sys 
import os
#os.chdir('../')
script_wkdir = os.getcwd()
sys.path.append(script_wkdir)


from salam.module_MD.Metrics import get_intersection_plus_smiles
from salam.module_MD.Metrics import get_centroid
# from salam.module_MD.Metrics import compute_similarity_matrix
from salam.module_MD.Metrics import map_molslibs_by_similarity
from salam.module_MD.Metrics import draw_mols_and_show

from salam.module_MD.Metrics import load_mols_in_sdf


#-----------------------------------------------------------------------------
#---- main here --------------------------------------------------------------
if __name__ == "__main__":
    print("Begin of program.")
    
    # -------- two steps -----------------------------------------------------
    # smis1 = [ 'CCCNCCO', 'CCNCCO', 'CC', 'COOC', 'CCCOOC', \
    #             'CCCO', 'CCCCO', 'CCOC', 'CCO', 'COC',\
    #                 'CCOOC', 'CCCCOC', 'CCC', 'CCCC', 'CNCCO'
    #               ]
        
    # smis2 = [ 'CCCNCCO', 'CCNCCO', 'CCCCOOC', 'CCCNCOC', 'CNCCOC',\
    #             'CC', 'CCCOOC', 'CCCNCCOC', 'CCCO', 'CCNCCOC',\
    #                 'CCCCO', 'CCO', 'CCNCOC', 'CCOOC', 'CCC',\
    #                     'CCCC', 'CNCOC', 'CNCCO' 
    #                   ]
        
    print("-"*60) 
    mutation_generation = int( GXXXX[1:] )
    GXXXX_PLUS1 = "G%04d"%(mutation_generation - 1)
    GXXXX_sdf_path1 = './project/'  +  GXXXX    +  '/'  + GXXXX  +  '.sdf'
    GXXXX_sdf_path2 = './project/'  +  GXXXX_PLUS1    +  '/'  + GXXXX_PLUS1  +  '.sdf'
    
    print("Loading sdf for FPSimilarity map, the path1 is:\n", GXXXX_sdf_path1)

    _ , smis1 = load_mols_in_sdf(sdf_path=GXXXX_sdf_path1)

    centriod_smi1, inner_max_ave_sim1 = get_centroid(smis=smis1)
    
    draw_mols_and_show([ centriod_smi1 ],
                        figname='./project/%s/images/foo--mols1-centriod.png'%GXXXX,
                        molsPerRow=1,
                        subImgSize=(800, 800)
                        )

    print('-'*60)
    if (mutation_generation > 0):
        print("Loading sdf for FPSimilarity map, the path2 is:\n", GXXXX_sdf_path2)        
        _ , smis2 = load_mols_in_sdf(sdf_path=GXXXX_sdf_path2)

        centriod_smi2, inner_max_ave_sim2 = get_centroid(smis=smis2)
        
        # draw_mols_and_show([ centriod_smi2 ],
        #                     figname='./project/%s/images/foo--mols1-centriod.png'%GXXXX_PLUS1,
        #                     molsPerRow=1,
        #                     subImgSize=(800, 800)
        #                     ) 


        intersection_smis, smis1_rev, smis2_rev, exchange_flag = get_intersection_plus_smiles(smis1=smis1, 
                                                                                              smis2=smis2
                                                                                              )
    
        new_smis1, new_smis2, mat_sim, inter_max_app_sim = map_molslibs_by_similarity(smis1=smis1_rev, 
                                                                                        smis2=smis2_rev,
                                                                                        fpSize=2048,
                                                                                        is_return_max_app_sim=True
                                                                                        ) 
    
    
        # draw_mols_and_show(new_smis1,
        #                     figname='./project/%s/images/foo--FPSmap-mols1.png'%GXXXX,
        #                     molsPerRow=3,
        #                     subImgSize=(800, 800),
        #                     num_of_mols=9
        #                     )
    
        # draw_mols_and_show(new_smis2,
        #                     figname='./project/%s/images/foo--FPSmap-mols2.png'%GXXXX,
        #                     molsPerRow=3,
        #                     subImgSize=(800, 800),
        #                     num_of_mols=9
        #                     )
    
        print("len(smis1) = ", len(smis1))
        print("len(smis2) = ", len(smis2))    
        print("exchange_flag = ", exchange_flag)
        print("len(smis1_rev) = ", len(smis1_rev))
        print("len(smis2_rev) = ", len(smis2_rev))
        print("len(intersection_smis) = ", len(intersection_smis))
    
        print("\ncentriod_smi1:\n", centriod_smi1)
        print("centriod_smi2:\n", centriod_smi2)
        
        print("inner_max_ave_sim1 = %10.3f"%inner_max_ave_sim1)
        print("inner_max_ave_sim2 = %10.3f"%inner_max_ave_sim2)
        
        print("inter_max_app_sim = %10.3f"%inter_max_app_sim)

        #----------- store data for mutation_diagram ---------------------------
        print("\nData for mutation_diagram is stored in /project/mutation_diagram.csv\n")
        with open('./project/mutation_diagram.csv', 'a+') as fr:
            fr.write('%5d'%mutation_generation + \
                     ', %s'%centriod_smi1 + \
                         ", %5d"%len(smis1) + \
                             ', %10.3f'%inner_max_ave_sim1 + \
                                 ', %10.3f'%inter_max_app_sim + "\n")

        
    else:
        print("mutation_generation = 0. Do not compute inter_max_app_sim.")
        print("set inter_max_app_sim =  1.000")
        inter_max_app_sim =  1.000
        
        print("len(smis1) = ", len(smis1))
        print("\ncentriod_smi1:\n", centriod_smi1)
        print("inner_max_ave_sim1 = %10.3f"%inner_max_ave_sim1)       

        
        #----------- store data for muation_diagram ---------------------------
        print("\nData for mutation_diagram is stored in /project/mutation_diagram.csv\n")
        with open('./project/mutation_diagram.csv', 'w') as fr:
            fr.write("mutation_generation,centriod_smi1,len_smis1,inner_max_ave_sim1,inter_max_app_sim\n")
            fr.write('%5d'%mutation_generation + \
                     ', %s'%centriod_smi1 + \
                         ", %5d"%len(smis1) + \
                             ', %10.3f'%inner_max_ave_sim1 + \
                                 ', %10.3f'%inter_max_app_sim + "\n")
    
    

    print("\nEnd of PG015!\n")




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  8 09:20:07 2021

@author: tucy
"""


from rdkit import Chem
#from rdkit.Chem import AllChem
import subprocess
#import shutil
import os

import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--gs_opt_method', type=str, default="b3lyp")
parser.add_argument('--num_serviers', type=int, default=2)
parser.add_argument('--remote_wkdir', type=str, default="project")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")

args = parser.parse_args()

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX


print(args.gs_opt_method, type(args.gs_opt_method))
gs_opt_method = args.gs_opt_method

print("num_serviers: ", args.num_serviers, type(args.num_serviers))
NUM_SERVIERS = args.num_serviers

print(args.remote_wkdir, type(args.remote_wkdir))
remote_wkdir = args.remote_wkdir


#-------- remote servier working dir ------------------------------------------
remote_base_wkdir = 'tucy@node130:/home/tucy/Computation-node131/Gaussian16-jobs/SALAM/%s/'%remote_wkdir
remote_GXXXX_opt_jobs_dir = remote_base_wkdir  + GXXXX +  "/opt_jobs/" 

print("The remote_GXXXX_opt_jobs_dir:")
print(remote_GXXXX_opt_jobs_dir)


base_wkdir = './project/'  + GXXXX  + '/'
picmols_sdffile_path = base_wkdir +  PXXXX  + '.sdf'

ms = [x for x in Chem.SDMolSupplier(picmols_sdffile_path)]

print("1st len(ms) = ", len(ms))

while ms.count(None): 
    ms.remove(None)

print("2nd len(ms) = ", len(ms))

call_cp_pg003_sh_modelcom_to_pick_mols_dir = 'cp   ./inputScripts/PG003__bat_sdf_to_gscom.sh   ./inputScripts/bat*.sh    ./inputScripts/g16*.sh    ./inputScripts/*-com  '  +  '   ./project/'  +  GXXXX  + '/pick_mols/  '
status, output = subprocess.getstatusoutput( call_cp_pg003_sh_modelcom_to_pick_mols_dir )
print("Copy inputScripts files. status, output = ", status, output)

# res = subprocess.check_output(["echo", "Hello World!"])
# print(res)


#------------------------------------------------------------------------------
base_wkdir = os.getcwd()
print("base_wkdir: \n", base_wkdir)

GXXXX_pick_mols_dir = './project/'  +  GXXXX +  '/pick_mols/'
os.chdir( GXXXX_pick_mols_dir )
CWD = os.getcwd()
print("CWD: \n", CWD)


status, output = subprocess.getstatusoutput(' cp   ./sdf-files/cpd-*.sdf   ./ ')
print("Copy sdf files. status, output = ", status, output)


print("Run ./PG003__bat_sdf_to_gscom.sh   gs-opt-model-com \n")

bat_sdf_to_gscom_script = "./PG003__bat_sdf_to_gscom.sh    "
b3lyp_gs_opt_model_com = "./b3lyp-gs-opt-model-com"
pm6d3_gs_opt_model_com = "./pm6d3-gs-opt-model-com"
pm7_gs_opt_model_com = "./pm7-gs-opt-model-com"


call_bat_sdf_to_gscom_script_b3lyp = bat_sdf_to_gscom_script + b3lyp_gs_opt_model_com
call_bat_sdf_to_gscom_script_pm6d3 = bat_sdf_to_gscom_script + pm6d3_gs_opt_model_com
call_bat_sdf_to_gscom_script_pm7 = bat_sdf_to_gscom_script + pm7_gs_opt_model_com

if gs_opt_method == "b3lyp":
    retcode = subprocess.call(call_bat_sdf_to_gscom_script_b3lyp, shell=True)
    print(retcode)
elif gs_opt_method == "pm6d3":
    retcode = subprocess.call(call_bat_sdf_to_gscom_script_pm6d3, shell=True)
    print(retcode)
elif gs_opt_method == "pm7":
    retcode = subprocess.call(call_bat_sdf_to_gscom_script_pm7, shell=True)
    print(retcode)
else:
    print("parameter: gs_opt_method, must be either b3lyp, pm6d3 or pm7")
    exit(1)

status, output = subprocess.getstatusoutput(' cp   ./bat*.sh   ./g16*.sh   ./*-com    ./com-files ')
print("Copy PBS Scripts files. status, output = ", status, output)


os.chdir('./com-files/')
print("Enter ./com-files/")

#-------- copy opt coms to  opt_jobs dir --------------------------------------
status, output = subprocess.getstatusoutput(' mkdir   ./opt_jobs   2>/dev/null ')
print("mkdir opt_jobs. status, output = ", status, output)

status, output = subprocess.getstatusoutput(' cp    ./bat*.sh    ./g16*.sh   ./cpd*.com    ./*-com   ./opt_jobs/   2>/dev/null')
print("Copy PBS Scripts files and cpd*.com to  opt_jobs.  status, output = ", status, output)


if (NUM_SERVIERS == 2):
    print("NUM_SERVIERS = ", NUM_SERVIERS)
    os.chdir('./opt_jobs/')
    print("Enter ./opt_jobs/")
    
    status, output = subprocess.getstatusoutput(' scp  cpd*.com  ' + remote_GXXXX_opt_jobs_dir )
    print("scp cpd*.com to remote_GXXXX_opt_jobs_dir. status, output = ", status, output)
else:
    print("NUM_SERVIERS = ", NUM_SERVIERS)

os.chdir(base_wkdir)
#os.chdir(script_call_path)
CWD = os.getcwd()
print("CWD: \n", CWD)
#------------------------------------------------------------------------------


print("\n### End of program: PG003!\n")




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar  1 13:53:00 2021

Using scikit-learn with RDKit

@author: tucy
"""

from salam.module_MD.D_Pi_A_Enumeration import D_A_Enumeration_new
from salam.module_MD.D_Pi_A_Enumeration import draw_mols

from rdkit import Chem
from rdkit.Chem import AllChem

from tqdm import tqdm
from salam.module_MD.D_Pi_A_Enumeration import try_embedmolecule



#--------------------------------------------------------------------------
def print_numAtoms_of_fragments(frags_suppl, frags_suppl_name='frags'):
    print("-"*60)    
    print("length of %s is : " % (frags_suppl_name), len(frags_suppl))
    i = 0
    for mol in frags_suppl:
        i += 1
        if mol is None: continue
        print("NumAtoms of frag %d is: " %(i) , mol.GetNumAtoms())


#--------------------------------------------------------------------------
#----- reading smiles of Donors, Acceptors from smi files -----------------

donor_suppl = Chem.SmilesMolSupplier('./data/Donor_smiles.smi')
#print_numAtoms_of_fragments(donor_suppl, "donor_suppl")
donor_mols = [x for x in donor_suppl if x is not None]
print("length of donor_mols is : ", len(donor_mols))

acceptor_suppl = Chem.SmilesMolSupplier('./data/Acceptor_smiles.smi')
#print_numAtoms_of_fragments(acceptor_suppl, "acceptor_suppl")
acceptor_mols = [x for x in acceptor_suppl if x is not None]
print("length of acceptor_mols is : ", len(acceptor_mols))


#----------------------------------------------------------------------------
#---- write molecules to SD File ------------------------------- 
donor_w = Chem.SDWriter('data/Donor_mols.sdf')
for m in donor_mols: 
    m_temp = Chem.AddHs(m)
    AllChem.EmbedMolecule(m_temp, randomSeed=38)
    donor_w.write(m_temp)

#---- write molecules to SD File ------------------------------- 
acceptor_w = Chem.SDWriter('data/Acceptor_mols.sdf')
for m in acceptor_mols: 
    m_temp = Chem.AddHs(m)
    AllChem.EmbedMolecule(m_temp, randomSeed=38)
    acceptor_w.write(m_temp)



# =============================================================================
# draw_mols(donor_mols, len(donor_mols), './images/foo001--donor-mols-preexperiment.png', molsPerRow=4, size=(800, 800))
# draw_mols(acceptor_mols, len(acceptor_mols), './images/foo001--acceptor-mols-preexperiment.png', molsPerRow=4, size=(800, 800))


#-----------------------------------------------------------------------------
#---- create combinational compound library --------------------------
#---- via calling D_A_Enumeration_new() or D_Pi_A_Enumeration_new()
ligandA_smis = [Chem.MolToSmiles(mol)  for mol in donor_mols]
ligandB_smis = [Chem.MolToSmiles(mol)  for mol in acceptor_mols] 


DA_mols = D_A_Enumeration_new(ligandA_smis, 
                              ligandB_smis, 
                              verbse=False)


All_mols = DA_mols


draw_mols(DA_mols, 9, './images/foo001--DA-mols-preexperiment.png', molsPerRow=3, size=(800, 800))


print("length of DA_mols is : ", len(DA_mols))
print("length of All_mols = DA_mols + DPiA_mols is : ", len(All_mols))



# #---- write molecules to SD File ------------------------------- 
# DA_mols_w = Chem.SDWriter('data/Donor_Acceptor_mols.sdf')
# for m in tqdm(DA_mols): 
#     m_temp = Chem.AddHs(m)
#     m1_temp, nb_try_EmbedMolecule = try_embedmolecule(m_temp)
#     DA_mols_w.write(m1_temp)
    

All_mols = All_mols[:1000] 

cpd_names = [ "cpd-%05d" % (i+1) for i in range(len(All_mols)) ]
for mol, cpd_name in zip(All_mols, cpd_names):
    mol.SetProp("_Name", cpd_name )


#---- write molecules to SD File ------------------------------- 
outfile_path = './project/G0000/G0000.sdf'
All_mols_w = Chem.SDWriter(outfile_path)
for m in tqdm(All_mols): 
    m_temp = Chem.AddHs(m)
    m1_temp, nb_try_EmbedMolecule = try_embedmolecule(m_temp)
    All_mols_w.write(m1_temp)



print("\nEnd of program PG001!\n")




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 25 14:30:21 2021

rdkit.SimDivFilters

@author: tucy
"""

import sys 
import os
#os.chdir('../')
script_wkdir = os.getcwd()
sys.path.append(script_wkdir)


import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")
parser.add_argument('--pickmols_size', type=int, default=100)

args = parser.parse_args()

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print("pickmols_size is ", args.pickmols_size, type(args.pickmols_size))
pickmols_size = args.pickmols_size


from rdkit import Chem
from rdkit.Chem.rdMolDescriptors import GetMorganFingerprint
from rdkit import DataStructs
from rdkit.SimDivFilters.rdSimDivPickers import MaxMinPicker

from salam.module_MD.D_Pi_A_Enumeration import draw_mols

from tqdm import tqdm
import datetime
import multiprocessing as mp

import subprocess
import shutil 
import time
from joblib import Parallel, delayed

from salam.module_MD.D_Pi_A_Enumeration import try_embedmolecule


#-----------------------------------------------------------------------------
def write_mol_to_sdf(mol, cpd_name, outfile_path='./project/G0000/separate_mols/'):
    
    outname=  outfile_path + '%s.sdf'%(cpd_name)
    #print(outname)
    one_w = Chem.SDWriter(outname)
    m_temp = Chem.AddHs(mol)
    m1_temp, nb_try_EmbedMolecule = try_embedmolecule(m_temp)
    m1_temp.SetProp("_Name", cpd_name)
    one_w.write(m1_temp)


def write_mols_paralell(mols, cpd_names, outfile_path='./project/G0000/separate_mols/'):
    #-------  paralell write mols-----------------------------------------------------------
    print("The outfile_path is : %s" % outfile_path )
    
    start = time.time()
    
    rm_GXXXX_separate_mols_sdf = 'rm -f   '  +  outfile_path + 'cpd-*.sdf  '
    (status0, output0) = subprocess.getstatusoutput( rm_GXXXX_separate_mols_sdf )
    
    Parallel(n_jobs=-1)(delayed(write_mol_to_sdf)(mol, cpd_name, outfile_path) \
                        for mol, cpd_name in zip(mols, cpd_names) )
    
    end = time.time()
    
    print('-'*60)
    print("write_mol_to_sdf run in paralell.")
    print('{:.4f} s'.format(end-start))



#-----------------main function -----------------------------------------------


#--- multiprocessing ---------------------------------------------------
num_cores = int(mp.cpu_count())
print("The number of processors of local machine is : " + str(num_cores) + " cores. ")
pool = mp.Pool(num_cores)

start_t0 = datetime.datetime.now()


#-------------------------------------------------------------------------


base_wkdir = './project/'  + GXXXX + "/"
allmols_sdffile_path = base_wkdir + GXXXX + ".sdf"

ms = [x for x in Chem.SDMolSupplier(allmols_sdffile_path)]

print("1st len(ms) = ", len(ms))

while ms.count(None): 
    ms.remove(None)

print("2nd len(ms) = ",len(ms))


#pick_ratio = 0.10
if len(ms) > 0:
    pick_ratio = pickmols_size / float( len(ms) )


#fps = [pool.apply_async(GetMorganFingerprint, args=(mol, 3) ) for mol in ms]
fps = [ GetMorganFingerprint(mol, 3) for mol in tqdm(ms)]
nfps = len(fps)

def distij(i, j, fps=fps):
    return 1-DataStructs.DiceSimilarity(fps[i], fps[j])

picker = MaxMinPicker()
pickIndices = picker.LazyPick(distij, nfps, int(nfps*pick_ratio), seed=23)
#pickIndices = pool.apply_async(picker.LazyPick, 
#                               args=(distij, nfps, int(nfps*pick_ratio)))  


pick_mols = [ms[x] for x in pickIndices]

print("len(ms): ", len(ms))
print("len(pick_mols): ", len(pick_mols))


pick_cpd_names = [mol.GetProp("_Name") for mol in pick_mols]

src_pick_cpds = [ base_wkdir + 'separate_mols/' + cpdname + '.sdf'  for cpdname in pick_cpd_names]


call_mkdir_separate_mols_pick_mols = 'mkdir     ./project/%s'%GXXXX  +  '/pick_mols/   '  +   '   ./project/%s'%GXXXX + '/separate_mols  ' +   '   ./project/%s'%GXXXX + '/images/   '  +  '    2> /dev/null '
status, output = subprocess.getstatusoutput( call_mkdir_separate_mols_pick_mols )
print("status, output = ", status, output)


call_rm_sdf_files = 'rm -f   ./project/%s'%GXXXX  +  '/pick_mols/cpd*.sdf  '  +   '   ./project/%s'%GXXXX + '/%s'%PXXXX +  '.sdf   2> /dev/null '
status, output = subprocess.getstatusoutput( call_rm_sdf_files )
print("status, output = ", status, output)


all_cpd_names = [mol.GetProp("_Name") for mol in ms]


if (GXXXX == "G0000"):
    print("split total sdf file to separate mols for mutation_generation = 0 only.")
    write_mols_paralell(ms, all_cpd_names, outfile_path='./project/%s/separate_mols/'%GXXXX )
else:
    print("\nDo not create dir ./project/%s/separate_mols/\n"%GXXXX )


dst_file_path = './project/%s/pick_mols/'%GXXXX

print("The src_pick_cpds is:\n",  dst_file_path)
print('-'*60)
for ele in tqdm(src_pick_cpds):
    print(ele)
    shutil.copy(ele, dst_file_path)


cat_sdfs_to_onefile = 'cat  ./project/%s'%GXXXX  +  '/pick_mols/cpd*.sdf  >>  '  +  './project/%s'%GXXXX  + '/%s'%PXXXX  +  '.sdf '
status, output = subprocess.getstatusoutput( cat_sdfs_to_onefile )
print("status, output = ", status, output)


#----------------------------------------------------------------
draw_mols(pick_mols[:16], 
          16, 
          './project/%s/images/foo--pick-mols-in-all-mols.png'%GXXXX, 
          molsPerRow=4, 
          size=(800, 800)
          )


end_t0 = datetime.datetime.now()
elapsed_sec0 = (end_t0 - start_t0).total_seconds()
print("\nTotal running time is : " + "{:.2f}".format(elapsed_sec0) + " sec")



print("\n### End of program PG002!\n")

#End of program



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb  7 09:15:51 2022

@author: tucy
"""


import argparse

parser = argparse.ArgumentParser(description="manual to this script:")
# parser.add_argument('--analyzed_property', type=str, default="tadf")
parser.add_argument('--energy_threshold1', type=float, default=2.00)
parser.add_argument('--energy_threshold2', type=float, default=2.50)
parser.add_argument('--stokes_shift', type=float, default=0.30)
parser.add_argument('--energy_tendency', type=str, default="descending")
parser.add_argument('--GXXXX', type=str, default="G0000")
parser.add_argument('--PXXXX', type=str, default="P0000")
parser.add_argument('--OXXXX', type=str, default="O0000")

args = parser.parse_args()

print(args.GXXXX, type(args.GXXXX))
print(args.PXXXX, type(args.PXXXX))
print(args.OXXXX, type(args.OXXXX))
GXXXX = args.GXXXX
PXXXX = args.PXXXX
OXXXX = args.OXXXX

print("stokes_shift: ", args.stokes_shift, " , type(args.stokes_shift): ", type(args.stokes_shift) )
STOKES_SHIFT = args.stokes_shift

print("energy_threshold1: ", args.energy_threshold1, " , type(args.energy_threshold1): ", type(args.energy_threshold1) )
ENERGY_THRESHOLD1 = args.energy_threshold1 + STOKES_SHIFT

print("energy_threshold2: ", args.energy_threshold2, " , type(args.energy_threshold2): ", type(args.energy_threshold2) )
ENERGY_THRESHOLD2 = args.energy_threshold2 + STOKES_SHIFT


print("args.energy_tendency: ", args.energy_tendency, " , type(args.energy_tendency): ", type(args.energy_tendency))
ENERGY_TENDENCY = args.energy_tendency

# print("analyzed_property: ", args.analyzed_property, type(args.analyzed_property) )
# analyzed_property = args.analyzed_property


import sys 
import os
#os.chdir('../')
script_wkdir = os.getcwd()
sys.path.append(script_wkdir)

# import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'
#import tensorflow as tf

import deepchem as dc
import numpy as np
import pandas as pd
#import tempfile
from sklearn.ensemble import RandomForestRegressor

from rdkit import Chem
#from rdkit.Chem import Draw, AllChem
#from rdkit.Chem import rdDepictor

from sklearn.model_selection import GridSearchCV
#from sklearn.metrics import mean_squared_error
from sklearn.model_selection import cross_val_score
import joblib
from sklearn.decomposition import PCA
from sklearn.preprocessing import MinMaxScaler
import matplotlib.pyplot as plt

#pd.set_option('display.max_columns', 100)
#pd.set_option('max_colwidth', 400)

from sklearn.impute import SimpleImputer
import subprocess

from salam.module_MD.D_Pi_A_Enumeration import draw_mols
from salam.module_MD.Metrics import getmaxminidxs_lt_le_gt_ge_criticval_df

#------------------------------------------------------------------------------
def display_scores(scores):
    print("Scores:", scores)
    print("Mean:", scores.mean())
    print("Standard deviation:", scores.std())


def load_mols_in_sdf(sdf_path='./data/Pick_mols.sdf'):
    #-------- loading  sdf file ---------------------------------------------------
    sdsuppl = Chem.SDMolSupplier(sdf_path)
    mols = [x for x in sdsuppl if x is not None]
    smiles = [Chem.MolToSmiles(mol) for mol in mols]
    print("-"*60)  
    print( "number of molecules is ", len(mols) )
    
    return mols, smiles


def impute_dataset(X, strategy="median"):
    imputer = SimpleImputer(strategy=strategy)
    imputer.fit(X)
    X_imputed = imputer.transform(X)
    print("type(X_imputed) is ", type(X_imputed))
    
    return X_imputed 



def load_tadf_csv_creat_properties_list(filename='./data/tadf-summary.csv'):
    #---load properties summary file-----------------------------------------------
    df_props = pd.read_csv(filename)
    print(df_props.head())
    df_props_num = df_props.drop("compound", axis=1)
    print(df_props_num.head())
    imputer = SimpleImputer(strategy="median")
    imputer.fit(df_props_num)
    props_imputed = imputer.transform(df_props_num)
    print("type(props_imputed) is ", type(props_imputed))
    
    column_names = ["T1_en", "S1_en", "S1_freq", "T2_en", "S2_en", "S2_freq",\
                    "T3_en","S3_en","S3_freq",\
                        # "T4_en","S4_en","S4_freq",\
                        # "T5_en","S5_en","S5_freq","T6_en","S6_en","S6_freq",\
                        #     "T7_en","S7_en","S7_freq","T8_en","S8_en","S8_freq",\
                        #     "T9_en","S9_en","S9_freq","T10_en","S10_en","S10_freq"
                    ]
    
    df_props_imputed = pd.DataFrame(data=props_imputed, columns=column_names)
    
    df_props_imputed["deltaEST"]  = df_props_imputed["S1_en"]  -  df_props_imputed["T1_en"] 
    
    properties = df_props_imputed["deltaEST"].tolist()

    print("-"*60) 
    print("properties set is")
    print(properties)
    
    print("-"*60) 
    print( "df_props_imputed.deltaEST.describe()" )
    print( df_props_imputed["deltaEST"].describe() )
    
    return properties



def load_tadf_csv_creat_properties_list_S1Energy(filename='./data/tadf-summary.csv'):
    #---load properties summary file-----------------------------------------------
    df_props = pd.read_csv(filename)
    print(df_props.head())
    df_props_num = df_props.drop("compound", axis=1)
    print(df_props_num.head())
    imputer = SimpleImputer(strategy="median")
    imputer.fit(df_props_num)
    props_imputed = imputer.transform(df_props_num)
    print("type(props_imputed) is ", type(props_imputed))
    
    column_names = ["T1_en", "S1_en", "S1_freq", "T2_en", "S2_en", "S2_freq",\
                    "T3_en","S3_en","S3_freq",\
                        # "T4_en","S4_en","S4_freq",\
                        # "T5_en","S5_en","S5_freq","T6_en","S6_en","S6_freq",\
                        #     "T7_en","S7_en","S7_freq","T8_en","S8_en","S8_freq",\
                        #     "T9_en","S9_en","S9_freq","T10_en","S10_en","S10_freq"
                    ]
    
    df_props_imputed = pd.DataFrame(data=props_imputed, columns=column_names)

    # df_props_imputed["deltaEST"]  = df_props_imputed["S1_en"]  -  df_props_imputed["T1_en"] 
    # properties = df_props_imputed["deltaEST"].tolist()
    properties = df_props_imputed["S1_en"].tolist()

    print("-"*60) 
    print("properties set is")
    print(properties)
    
    print("-"*60) 
    print( "df_props_imputed.S1_en.describe()" )
    print( df_props_imputed["S1_en"].describe() )
    
    return properties



def creat_ECFP_RDKITDES_CM_features_dataset(mols, smiles, properties, is_compute_CM=True):   
    #--------- ECFP ---------------------------------------------------
    featurizer = dc.feat.CircularFingerprint(size=2048)
    ecfp = featurizer.featurize(smiles)
    dataset = dc.data.NumpyDataset(X=ecfp, y=np.array(properties))
    print("ecfp.shape is ", ecfp.shape )
    print("len(dataset) is ", len(dataset) ) 
    
    #-------------------------------------------------------------------
    #------- RDKitDescriptors ------------------------------------------
    rdkit_featurizer = dc.feat.RDKitDescriptors()
    rdkit_des_features = rdkit_featurizer(smiles)
    dataset1 = dc.data.NumpyDataset(X=rdkit_des_features, y=np.array(properties))
    print("len(dataset1) is ", len(dataset1) ) 
    
    print("-"*60) 
    print("RDKit Descriptors and values for a compound.") 
    for feature, descriptor in zip(rdkit_des_features[0][:], rdkit_featurizer.descriptors):
        print(descriptor, feature)
    
    print('The number of sample present is: ', len(rdkit_des_features))
    print('The number of descriptors present is: ', len(rdkit_des_features[0]))
    
    print("rdkit_des_features.shape is ", rdkit_des_features.shape)
    
    #------- WeaveFeaturizer and MolGraphConvFeaturizer
    
    #------------------------------------------------------------------------------
    #------- CoulombMatrix --------------------------------------------------------
    if (is_compute_CM == True):
        coulomb_mat = dc.feat.CoulombMatrix(max_atoms=100)
        coulomb_mat_features = coulomb_mat(mols)
        
        print("type(coulomb_mat_features) ", type(coulomb_mat_features))
        print("coulomb_mat_features.shape is ", coulomb_mat_features.shape)
        
        coulomb_mat_features = coulomb_mat_features.reshape(coulomb_mat_features.shape[0],
                                                            coulomb_mat_features.shape[1]*coulomb_mat_features.shape[2])
        
        print("coulomb_mat_features.shape is ", coulomb_mat_features.shape)
        
        dataset2 = dc.data.NumpyDataset(X=coulomb_mat_features, y=np.array(properties))
        print("len(dataset2) is ", len(dataset2) ) 
        
        return  dataset, dataset1, dataset2
    else:
        return  dataset, dataset1


def creat_smiles_properties_csv_file(smiles, properties, file_path='./data/smiles_properties.csv'):
    #----------- CSV DataLoader ---------------------------------------------------
    # make a dataframe object for creating a CSV file
    df = pd.DataFrame(list(zip(smiles, properties)), columns=["SMILES", "property"])
    print(df)
    tmp_path = file_path
    with open(tmp_path, 'w') as tmp:
        df.to_csv(tmp)



def gridsearchcv_evaluate_model(dataset, model_path=None, verbs=False):
    print("len(dataset) is ", len(dataset) )
    
    X_train, y_train = dataset.X, dataset.y
    
    param_grid = [
        {'n_estimators': [3, 10, 30, 100], 
         # 'criterion': ["mse", "mae"],
         'criterion': ["squared_error", "absolute_error"],
         'max_depth': [2, 5, 10, 50],
         'max_features': ["auto", "sqrt", "log2"]},
        {'bootstrap': [False], 
         'n_estimators': [3, 10, 30, 100], 
         # 'criterion': ["mse", "mae"],
         'criterion': ["squared_error", "absolute_error"],
         'max_depth': [2, 5, 10, 50],
         'max_features': ["auto", "sqrt", "log2"]},
    ]
    
    forest_reg = RandomForestRegressor(random_state=0)
    
    grid_search = GridSearchCV(forest_reg, 
                               param_grid, 
                               cv=5,
                               scoring='neg_mean_squared_error',
                               return_train_score=True,
                               n_jobs=-1,
                               )
    
    grid_search.fit(X_train, y_train)
    
    if verbs == True:
        print('-'*60)
        cvres = grid_search.cv_results_
        for mean_score, params in zip(cvres["mean_test_score"], cvres["params"]):
            print(np.sqrt(-mean_score), params)
    
    print('-'*60)
    print("grid_search.best_params_ is: \n", grid_search.best_params_)
    print("grid_search.best_estimator_ is: \n", grid_search.best_estimator_)
    
    best_rgr = grid_search.best_estimator_
    
    best_rgr.fit(X_train, y_train)
    
    
    scores = cross_val_score(best_rgr, 
                             X_train, 
                             y_train,
                             scoring="neg_mean_squared_error", 
                             cv=5)
    tree_rmse_scores = np.sqrt(-scores)
    
    print('-'*60)
    display_scores(tree_rmse_scores)
    
    if model_path is not None:
        joblib.dump(best_rgr, model_path)
    #joblib.dump(best_rgr, "./models/best_rgr.pkl")
    

def pca_decomposition_visualize(dataset, figname1=None, xlims=None, ylims=None):
    #-------PCA for visualization -------------------------------------------------
    X = dataset.X.copy()
    y = dataset.y.copy()
    pca = PCA(n_components=2, svd_solver='auto')
    X_pca = pca.fit_transform(X)
    
    mm_scaler = MinMaxScaler()
    X_pca_scaled = mm_scaler.fit_transform(X_pca)
    
    print('-'*60)
    print(pca.explained_variance_ratio_)
    print(pca.singular_values_)
    print(X.shape)
    print(X_pca.shape)
    
    
    df_pca = pd.DataFrame(data=X_pca_scaled, columns=["1st-component", "2nd-component"])
    df_pca["property"] = y
    
    ax1 = df_pca.plot(kind="scatter", 
                x="1st-component", 
                y="2nd-component", 
                alpha=0.4,
                #label="prop", 
                figsize=(10,6),
                c="property", 
                cmap=plt.get_cmap("jet"), 
                colorbar=True,
    )
    #plt.legend()
    
    if xlims is not None:
        left, right = xlims
        plt.xlim(left, right)
        ax1.set_xticks([left, right])
    
    if ylims is not None:
        bottom, top = ylims
        plt.ylim(bottom, top)    
        ax1.set_yticks([bottom, top])
    
    plt.xlabel('1st-component')
    plt.ylabel('2nd-component')
    
    fig1 = ax1.get_figure() 
    
    fig1.savefig(figname1)
    
    #plt.show()
    #plt.pause(10)
    plt.close('all')
    

#------ load and predict ------------------------------------------
def load_sdf_to_df(sdf_path='./data/All_mols.sdf'):
    #-------- loading  sdf file ---------------------------------------------------
    sdsuppl = Chem.SDMolSupplier(sdf_path)
    #sdsuppl = Chem.SDMolSupplier('./data/Prod_mols.sdf')
    
    mols = [x for x in sdsuppl if x is not None]
    smiles = [Chem.MolToSmiles(mol) for mol in mols]
    compoundnames = [mol.GetProp("_Name") for mol in mols]
    properties = [np.nan for i in range(len(mols))]
    print("-"*60)  
    print( "len(mols) is ", len(mols) ) 
    print("len(properties) is ", len(properties))
    
    df_mols = pd.DataFrame(data=zip(mols, smiles, compoundnames), columns=["mols", "smiles", "compoundnames"])
    
    df_mols.set_index("compoundnames", inplace=True)
    
    print(df_mols.head())
    
    return mols, smiles, compoundnames, properties, df_mols


def load_and_predict(df, dataset, model_path=None, ascending=False):

    if model_path is not None:
        model_loaded = joblib.load(model_path)
    else:
        print("loading model %s failed!" % model_path)
        exit(-1)

    # model evaluate on  all molecules. 
    prop_preds = model_loaded.predict(dataset.X)
    print( "len(prop_preds) is ", len(prop_preds) )
    
    df["prop_preds"] = prop_preds
    
    df.sort_values(by="prop_preds", ascending=ascending, inplace=True)

    print('# The optimal 16 molecules are: ')
    print( df.head(16) )
    
    return df


def draw_optimal_mols(df, imgname='./images/foo009-optimal-mols.png', unit=' a.u. '):
    opt_mols = df["mols"].tolist()[:16]
    props_values = df["prop_preds"].tolist()[:16] 
    props_as_legends = [" " + str(round(number, 3)) + unit for number in props_values]
    
    draw_mols(opt_mols, 
              16, 
              imgname, 
              molsPerRow=4, 
              size=(800, 800),
              legends=props_as_legends,
              )


def plot_prop_pred(df, figname='./images/foo009-property-statistics.png', xlabel_name='Energy gap'):
    ax1 = df['prop_preds'].plot(title='Distribution of property',
                                  kind='hist',
                                )
    
    plt.xlabel(xlabel_name)
    #plt.ylabel('Frequency')
    
    fig1 = ax1.get_figure() 
    fig1.savefig(figname)
    #plt.show()
    #plt.pause(10)
    plt.close()



#----MAIN Function begins -----------------------------------------------------
#------------------------------------------------------------------------------

Model_Path1='./project/%s/models/ECFP/rf_reg_model_ECFP--S1En.pkl'%GXXXX
Model_Path2='./project/%s/models/RDKIT_DES/rf_reg_model_RDKIT_DES--S1En.pkl'%GXXXX

if  not(os.path.isfile( Model_Path1 ) and os.path.isfile( Model_Path2 ) ):
    
    #------- featurization and trainning of ML model -------------------------------
    #mols, smiles = load_mols_in_sdf(sdf_path='./data/Pick_mols.sdf')
    
    GXXXX_PXXXXrev_sdf_path = './project/'  +  GXXXX  + '/'    +  PXXXX   +  'rev.sdf'
    mols, smiles = load_mols_in_sdf(sdf_path=GXXXX_PXXXXrev_sdf_path) 
    
    #properties = load_nlo_csv_creat_properties_list(filename='./data/nlo-summary.csv')
    #properties = load_tadf_csv_creat_properties_list(filename='./data/tadf-summary.csv')
    
    
    GXXXX_tadf_summary_PXXXXrev_csv_path = './project/'   +  GXXXX  + '/tadf_summary_'  +  PXXXX  +  '.csv'
    properties = load_tadf_csv_creat_properties_list_S1Energy(filename=GXXXX_tadf_summary_PXXXXrev_csv_path)
    
    
    #dataset, dataset1, dataset2 =  creat_ECFP_RDKITDES_CM_features_dataset(mols, smiles, properties)
    dataset, dataset1 =  creat_ECFP_RDKITDES_CM_features_dataset(mols, 
                                                                  smiles, 
                                                                  properties, 
                                                                  is_compute_CM=False
                                                                  )
    
    X_imputed = impute_dataset(dataset.X, strategy="median")
    X1_imputed = impute_dataset(dataset1.X, strategy="median")
    # X2_imputed = impute_dataset(dataset2.X, strategy="median")
    
    dataset_imputed = dc.data.NumpyDataset(X=X_imputed, y=np.array(properties)) 
    dataset1_imputed = dc.data.NumpyDataset(X=X1_imputed, y=np.array(properties)) 
    # dataset2_imputed = dc.data.NumpyDataset(X=X2_imputed, y=np.array(properties)) 
    
    creat_smiles_properties_csv_file(smiles, 
                                      properties,
                                      file_path='./project/%s/smiles_properties_S1En_%s.csv'%(GXXXX, PXXXX)
                                      )
    
    call_mkdir_GXXXX_models_dir = 'mkdir  ./project/'  +  GXXXX  +  '/models/  2>/dev/null '
    status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_models_dir )
    print("mkdir  ./project/%s/models/.  status, output = "%(GXXXX), status, output)
    
    call_mkdir_GXXXX_models_ECFP_dir = 'mkdir  ./project/'  +  GXXXX  +  '/models/ECFP/  2>/dev/null '
    status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_models_ECFP_dir )
    print("mkdir  ./project/%s/models/ECFP/.  status, output = "%(GXXXX), status, output)
    
    call_mkdir_GXXXX_models_RDKIT_DES_dir = 'mkdir  ./project/'  +  GXXXX  +  '/models/RDKIT_DES/  2>/dev/null '
    status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_models_RDKIT_DES_dir )
    print("mkdir  ./project/%s/models/RDKIT_DES/.  status, output = "%(GXXXX), status, output)
    
    # status, output = subprocess.getstatusoutput('mkdir  ./project/G0000/models/CM_S1En/  2>/dev/null ')
    # print("mkdir  ./project/G0000/models/CM_S1En/.  status, output = ", status, output)
    
    
    #------------------------------------------------------------------------------
    print("The ML models are stored at: " + "./project/"  +  GXXXX + "/models/" + "\n" )
    print("ECFP model", "-"*60) 
    rf_reg_ECFP = gridsearchcv_evaluate_model(dataset, 
                                              model_path='./project/%s/models/ECFP/rf_reg_model_ECFP--S1En.pkl'%GXXXX
                                              )
    
    
    print("RDKIT_DES model", "-"*60) 
    rf_reg_RDKIT_DES = gridsearchcv_evaluate_model(dataset1_imputed, 
                                                    model_path='./project/%s/models/RDKIT_DES/rf_reg_model_RDKIT_DES--S1En.pkl'%GXXXX
                                                    )
    
    
    # print("CM model", "-"*60) 
    # rf_reg_CM = gridsearchcv_evaluate_model(dataset2, 
    #                                         model_path='./project/G0000/models/CM/rf_reg_model_CM.pkl'
    #                                         )
    
    #------------------------------------------------------------------------------
    call_mkdir_GXXXX_images_dir = 'mkdir  ./project/'  +  GXXXX   + '/images/  2>/dev/null '
    status, output = subprocess.getstatusoutput( call_mkdir_GXXXX_images_dir )
    print("mkdir  ./project/%s/images/.  status, output = "%GXXXX, status, output)
    
    print("The PCA decomposition of X vector vs property is stored at: ")
    print( "./project/%s/images/"%GXXXX )
    
    fignamea = './project/%s/images/foo-props-S1En-pca-ECFP-1.png'%GXXXX
    print("save figure\n %s \n" %(fignamea))
    pca_decomposition_visualize(dataset, figname1=fignamea)
    
    
    fignameb = './project/%s/images/foo-props-S1En-pca-RDKITDES-1.png'%GXXXX
    print("save figure\n %s \n" %(fignameb))
    pca_decomposition_visualize(dataset1_imputed, figname1=fignameb)
    
    
    # fignamec = './project/G0000/images/foo-props-pca-CM-1.png'
    # print("save figure\n %s \n" %(fignamec))
    # pca_decomposition_visualize(dataset2, 
    #                             figname1=fignamec, 
    #                             #xlims=(0.0, 0.10), 
    #                             #ylims=(0.0, 0.05),
    #                             )

else:
    print("\nML model exist. Turn to next step. Load and Predict!\n")


#----load and predict whole lib of mols----------------------------------------
print("-"*60) 
GXXXX_sdf_path = './project/'  +  GXXXX    +  '/'  + GXXXX  +  '.sdf'
print("Loading sdf for ML prediction, the path is:\n", GXXXX_sdf_path)

mols, smiles, compoundnames, properties, df_mols = load_sdf_to_df(sdf_path=GXXXX_sdf_path )

dataset, dataset1 = creat_ECFP_RDKITDES_CM_features_dataset(mols, 
                                                            smiles, 
                                                            properties, 
                                                            is_compute_CM=False
                                                            )

print("The ECFP model is loaded for prediction of whole mols.")

X_imputed = impute_dataset(dataset.X, strategy="median")
dataset_imputed = dc.data.NumpyDataset(X=X_imputed, y=np.array(properties)) 

if (ENERGY_TENDENCY == "descending"):
    df_ECFP = load_and_predict(df_mols, 
                               dataset_imputed, 
                               model_path='./project/%s/models/ECFP/rf_reg_model_ECFP--S1En.pkl'%GXXXX, 
                               ascending=False
                               )
elif (ENERGY_TENDENCY == "ascending"):
    df_ECFP = load_and_predict(df_mols, 
                               dataset_imputed, 
                               model_path='./project/%s/models/ECFP/rf_reg_model_ECFP--S1En.pkl'%GXXXX, 
                               ascending=True
                               )
else:
    print("ENERGY_TENDENCY: descending or ascending.")    
    exit(1)


print("The optimal mols is shown in fig: \n", './project/%s/images/foo-ECFP-optimal-mols-in-all-mols--S1En.png'%GXXXX )
draw_optimal_mols(df_ECFP, 
                  imgname='./project/%s/images/foo-ECFP-optimal-mols-in-all-mols--S1En.png'%GXXXX, 
                  unit=' eV '
                  )

print("The property statistics is shown in fig: \n", './project/%s/images/foo-property-statistics-in-all-mols--S1En.png'%GXXXX )
plot_prop_pred(df_ECFP, 
               figname='./project/%s/images/foo-property-statistics-in-all-mols--S1En.png'%GXXXX, 
               xlabel_name='S1 Energy'
               )


#---- write df_ECFP to csv-----------------------------------------------------
print('-'*60)
print(df_ECFP.head() )
GXXXX_df_ECFP_csv_path = './project/'  +  GXXXX    +  '/'  + GXXXX   +  '_df_ECFP--S1En.csv'
call_rm_GXXXX_df_ECFP_csv_file =  'rm -f  '  +  GXXXX_df_ECFP_csv_path  +  '   2>/dev/null  '
status, output = subprocess.getstatusoutput( call_rm_GXXXX_df_ECFP_csv_file )
print("rm -f  %s.    status, output = "%GXXXX_df_ECFP_csv_path,  status, output)
print("df_ECFP is stored at %s\n"%GXXXX_df_ECFP_csv_path )

with open(GXXXX_df_ECFP_csv_path, 'w') as tmp:
    df_ECFP.to_csv(tmp)
#-----------------------------------------------------------------------------


df_ECFP.reset_index(drop=False, inplace=True)
print('-'*60)
print("After reset_index")
print(df_ECFP.head() )


#--- define  gt_idxmax_B for blue color. energy > ENERGY_THRESHOLD2 = 2.80 ev -------
#--- if gt_idxmax_B > 0, the indexs for blue mols are  [0 : gt_idmax_B]
#--- then, le_idxmin_G is the ending index for green color. 
lt_idxmin_G, le_idxmin_G, gt_idxmax_B, ge_idxmax_B = getmaxminidxs_lt_le_gt_ge_criticval_df(df_ECFP, 
                                                                                            colname='prop_preds',
                                                                                            critical_value=ENERGY_THRESHOLD2,
                                                                                            ascending=False)

#--- define  lt_idxmin_R for red color. energy < ENERGY_THRESHOLD1 = 2.30 ev -------
#--- if lt_idxmin_R > 0, the indexs for red mols are  [lt_idmin_R : ]
#--- then, ge_idxmax_G is the starting index for green color. 
lt_idxmin_R, le_idxmin_R, gt_idxmax_G, ge_idxmax_G = getmaxminidxs_lt_le_gt_ge_criticval_df(df_ECFP, 
                                                                                            colname='prop_preds',
                                                                                            critical_value=ENERGY_THRESHOLD1,
                                                                                            ascending=False)

if (gt_idxmax_B > 0):
    optimal_smis_B = df_ECFP.smiles.tolist()[0 : gt_idxmax_B]
    optimal_cpdnames_B = df_ECFP.compoundnames.tolist()[0 : gt_idxmax_B]
    prop_preds_S1En_B = df_ECFP.prop_preds.tolist()[0 : gt_idxmax_B]

    print("-"*60)
    # print("Blue mols, the optimal_cpdname, optimal_smis and prop_preds_S1En are:")
    # for cpdname, smi, S1En in zip(optimal_cpdnames_B, optimal_smis_B, prop_preds_S1En_B):
    #     print(cpdname, " , ", smi, " , %8.4f"%S1En )    
    
    print("Blue mols, final item, the optimal_cpdname, optimal_smis and prop_preds_S1En are:")
    print(optimal_cpdnames_B[-1], optimal_smis_B[-1], prop_preds_S1En_B[-1])

else:
    optimal_smis_B = set()
    print("Blue mols is empty!")



if (le_idxmin_G < df_ECFP.shape[0]) and (ge_idxmax_G < df_ECFP.shape[0]) and (le_idxmin_G < ge_idxmax_G):
    optimal_smis_G = df_ECFP.smiles.tolist()[le_idxmin_G : ge_idxmax_G]
    optimal_cpdnames_G = df_ECFP.compoundnames.tolist()[le_idxmin_G : ge_idxmax_G]
    prop_preds_S1En_G = df_ECFP.prop_preds.tolist()[le_idxmin_G : ge_idxmax_G]

    print("-"*60)
    # print("Green mols, the optimal_cpdname, optimal_smis and prop_preds_S1En are:")
    # for cpdname, smi, S1En in zip(optimal_cpdnames_G, optimal_smis_G, prop_preds_S1En_G):
    #     print(cpdname, " , ", smi, " , %8.4f"%S1En )    

    print("Green mols, final item, the optimal_cpdname, optimal_smis and prop_preds_S1En are:")
    print(optimal_cpdnames_G[-1], optimal_smis_G[-1], prop_preds_S1En_G[-1])

else:
    optimal_smis_G = set()
    print("Green mols is empty!")



if (lt_idxmin_R < df_ECFP.shape[0]):
    optimal_smis_R = df_ECFP.smiles.tolist()[lt_idxmin_R : ]
    optimal_cpdnames_R = df_ECFP.compoundnames.tolist()[lt_idxmin_R : ]
    prop_preds_S1En_R = df_ECFP.prop_preds.tolist()[lt_idxmin_R : ]

    print("-"*60)
    # print("Red mols, the optimal_cpdname, optimal_smis and prop_preds_S1En are:")
    # for cpdname, smi, S1En in zip(optimal_cpdnames_R, optimal_smis_R, prop_preds_S1En_R):
    #     print(cpdname, " , ", smi, " , %8.4f"%S1En )    

    print("Red mols, final item, the optimal_cpdname, optimal_smis and prop_preds_S1En are:")
    print(optimal_cpdnames_R[-1], optimal_smis_R[-1], prop_preds_S1En_R[-1])

else:
    optimal_smis_R = set()
    print("Red mols is empty!")


print("Size of Blue, Green, Red mols are: ", len(optimal_smis_B), len(optimal_smis_G), len(optimal_smis_R))




print("\n### End of program: PG014!\n")




__all__ = ["module_MD"]
def main():
    """Entry point for the application script"""
    #print("Call your main application code here.")
    print("The typical use of salam:")
    print("-"*30)
    print("#1. Uncompress the source tarbal salam-0-0-2.tar.gz to a dir e.g. salam-0-0-2/ ;")
    print("#2. cd salam-0-0-2/ ;")
    print("#3. Using venv to create a virtual environment with your python intepreter. preferably, python3.8 ;")
    print(" python3   -m  venv    ./ ; ")
    print("#4. Activate it ;")
    print(" source  ./bin/activate ;")
    print("#5. Install dependency pakages ;")
    print(" pip install rdkit ; preferably, rdkit==2024.3.2")
    print(" pip install salam; that will install most of the dependency packages, e.g., numpy, scipy, pandas, etc;")
    print(" or pip install --upgrade salam ;")
    print("#6. Verify if the installation is correct or not? ;")
    print(" ipython ")
    print(" import salam")
    print(" Alternatively, cp tests/ to ./, and run the test scripts.")
